@@ -1,5 +1,6 @@
 /* ====================================================================
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Ben Laurie.  All rights reserved.
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999, 2000 Ben Laurie.
+ *  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -114,6 +115,8 @@
 static const char *s_szCacheServerPort;
 static const char *s_szCacheServerDir;
 static char *s_szCacheServerPath;
+static char *s_szCacheServerArg1;
+static char *s_szCacheServerArg2;
 
 # define DEBUG_SESSIONS	FALSE
 
@@ -341,8 +344,15 @@ static void InfoCallback(SSL *s,int where,int ret)
 
 #endif /* DEBUG_SSL_STATE */
 
+typedef enum
+    {
+    RAND_EGD,
+    RAND_FILE
+    } RFType;
+
 typedef struct
     {
+    RFType eType;
     const char *szFile;
     int nBytes;
     } RandomFile;
@@ -353,7 +363,9 @@ static void ReadRandomFiles(server_rec *pServer,RandomFile *pRF,int nRF)
 
     for(n=0 ; n < nRF ; ++n,++pRF)
 	{
-	int r=RAND_load_file(pRF->szFile,pRF->nBytes);
+	int r=pRF->eType == RAND_EGD
+	  ? RAND_egd_bytes(pRF->szFile,pRF->nBytes)
+	  : RAND_load_file(pRF->szFile,pRF->nBytes);
 	ap_log_error(APLOG_MARK,APLOG_DEBUG|APLOG_NOERRNO,pServer,
 		     "Random input %s(%d) -> %d",pRF->szFile,pRF->nBytes,r);
 	}
@@ -1485,7 +1497,8 @@ static int CacheLauncher(void *cmd,child_info *pInfo)
 	}
     ap_log_error(APLOG_MARK,APLOG_DEBUG|APLOG_NOERRNO,s_pServer,
 		 "gcache pid is %d",getpid());
-    execl(cmd,"gcache",a1,s_szCacheServerPort,NULL);
+    execl(cmd,cmd,a1,s_szCacheServerPort,s_szCacheServerArg1,
+	  s_szCacheServerArg2,NULL);
     perror(cmd);
     assert(0);
     return 1;
@@ -1879,9 +1892,11 @@ static const char *SSLCacheServerPort(cmd_parms *cmd,char *struct_ptr,
     }
 
 static const char *SSLCacheServerPath(cmd_parms *cmd,char *struct_ptr,
-				      char *arg)
+				      char *arg,char *arg2,char *arg3)
     {
     s_szCacheServerPath=ap_server_root_relative(cmd->pool,arg);
+    s_szCacheServerArg1=arg2;
+    s_szCacheServerArg2=arg3;
 
     return NULL;
     }
@@ -1895,27 +1910,39 @@ static const char *SSLCacheServerDir(cmd_parms *cmd,char *struct_ptr,
     }
 #endif
 
+static void SetupRandomFile(RandomFile *prf,char *arg1,char *arg2,char *arg3)
+    {
+    if(!arg3)
+	{
+	arg3=arg2;
+	arg2=arg1;
+	arg1="file";
+	}
+
+    prf->eType=strcasecmp(arg1,"file") ? RAND_EGD : RAND_FILE;
+    prf->szFile=arg2;
+    prf->nBytes=atoi(arg3);
+    }
+
 static const char *SSLRandomFile(cmd_parms *cmd,char *struct_ptr,char *arg1,
-				 char *arg2)
+				 char *arg2,char *arg3)
     {
     RandomFile rf;
 
-    rf.szFile=arg1;
-    rf.nBytes=atoi(arg2);
+    SetupRandomFile(&rf,arg1,arg2,arg3);
     ReadRandomFiles(cmd->server,&rf,1);
 
     return NULL;
     }
 
 static const char *SSLRandomFilePerConnection(cmd_parms *cmd,char *struct_ptr,
-					      char *arg1,char *arg2)
+					      char *arg1,char *arg2,char *arg3)
     {
     SSLConfigRec *pConfig=ap_get_module_config(cmd->server->module_config,
 					       &apache_ssl_module);
     RandomFile *prf=ap_push_array(pConfig->aRandomFiles);
 
-    prf->szFile=arg1;
-    prf->nBytes=atoi(arg2);
+    SetupRandomFile(prf,arg1,arg2,arg3);
 
     return NULL;
     }
@@ -2033,16 +2060,16 @@ command_rec ssl_cmds[]=
     { "SSLSessionCacheTimeout",set_server_int_slot,
       (void *)XtOffsetOf(SSLConfigRec,tSessionCacheTimeout),RSRC_CONF,TAKE1,
       "Lifetime in seconds of session cache objects" },
-    { "SSLCacheServerPath",SSLCacheServerPath,NULL,RSRC_CONF,TAKE1,
-      "Path of the global cache server" },
+    { "SSLCacheServerPath",SSLCacheServerPath,NULL,RSRC_CONF,TAKE123,
+      "Path of the global cache server and up to two arguments" },
     { "SSLCacheServerRunDir",SSLCacheServerDir,NULL,RSRC_CONF,TAKE1,
       "Path for the server to run in (in case of coredumps)" },
 #endif
-    { "SSLRandomFile",SSLRandomFile,NULL,RSRC_CONF,TAKE2,
-      "A file to read randomness from and a maximum number of bytes to read" },
+    { "SSLRandomFile",SSLRandomFile,NULL,RSRC_CONF,TAKE23,
+      "A type (file or EGD), a file to read randomness from and a maximum number of bytes to read" },
     { "SSLRandomFilePerConnection", SSLRandomFilePerConnection,NULL,RSRC_CONF,
-      TAKE2,
-      "A file to read randomness from and a maximim number of bytes to read on each request" },
+      TAKE23,
+      "A type (file or EGD), a file to read randomness from and a maximim number of bytes to read on each request" },
     { "SSLLogFile",SSLLogFile,NULL,RSRC_CONF,TAKE1,
       "Defunct SSL log file - DO NOT USE" },
     { NULL }
